/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.model;

import hu.midori.kosmos.model.util.ModelUtils;
import hu.midori.kosmos.model.util.TimestampedEntity;
import hu.midori.kosmos.model.util.ZoomableImageLocator;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Wraps a <i>Subversion</i> repository.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class SvnRepository extends TimestampedEntity {
    /** SVN URL. */
    private String location;
    /** Latest revision, e.g. "184". */
    private long revision;

    /** Full history of the commits in reversed order. */
    public List<ScmRepositoryChange> commits;

    /** Number of commits since the repo was created. */
    private int commitsTotal;
    /** Number of commits today. */
    private int commitsToday;
    /** Number of commits over the last 7 days (one week). */
    private int commitsLast7Days;
    /** Number of commits over the last 31 days (about one month). */
    private int commitsLast31Days;

    /** Number of committers since the repo was created. */
    private int committersTotal;
    /** Number of committers today. */
    private int committersToday;
    /** Number of committers over the last 7 days (one week). */
    private int committersLast7Days;
    /** Number of committers over the last 31 days (about one month). */
    private int committersLast31Days;

    /** Locator of the files-per-file-type chart image. */
    private ZoomableImageLocator filesPerFileTypeChartLocator;
    /** Locator of the commits-per-author chart image. */
    private ZoomableImageLocator commitsPerAuthorChartLocator;
    /** Locator of the commits-per-file chart image. */
    private ZoomableImageLocator commitsPerFileChartLocator;
    /** Locator of the commits-per-week chart image. */
    private ZoomableImageLocator commitsPerWeekChartLocator;
    /** Locator of the repo-entries-per-week chart image. */
    private ZoomableImageLocator repoEntriesPerWeekChartLocator;

    /** Number of dirs in this repo. */
    private int dirs;
    /** Number of files in this repo. */
    private int files;
    /** Total size of files. */
    private int totalFileSize;

    /** Files-per-file-type sorted by files in descending order. */
    private List<Map.Entry<String,Integer>> filesPerFileType;
    /** Commits-per-author sorted by commits in descending order. */
    private List<Map.Entry<String,Integer>> commitsPerAuthor;
    /** Commits-per-file sorted by commits in descending order. */
    private List<Map.Entry<String,Integer>> commitsPerFile;

    /**
     * By design this should be private to avoid instantiation
     * without discriminator, but Hessian needs default ctor.
     */
    public SvnRepository() {
        super(null, null);
    }

    public SvnRepository(String location, long revision,
        Date dateCreated, List<ScmRepositoryChange> commits,
        int commitsTotal, int commitsToday, int commitsLast7Days, int commitsLast31Days,
        int committersTotal, int committersToday, int committersLast7Days, int committersLast31Days,
        ZoomableImageLocator filesPerFileTypeChartLocator, ZoomableImageLocator commitsPerAuthorChartLocator, ZoomableImageLocator commitsPerFileChartLocator, ZoomableImageLocator commitsPerWeekChartLocator, ZoomableImageLocator repoEntriesPerWeekChartLocator,
        int dirs, int files, int totalFileSize,
        List<Map.Entry<String,Integer>> filesPerFileType, List<Map.Entry<String,Integer>> commitsPerAuthor, List<Map.Entry<String,Integer>> commitsPerFile) {
        super(location + revision + commits.get(0).getDate(), dateCreated, commits.get(0).getDate());

        this.location = location;
        this.revision = revision;

        this.commits = commits;

        this.commitsTotal = commitsTotal;
        this.commitsToday = commitsToday;
        this.commitsLast7Days = commitsLast7Days;
        this.commitsLast31Days = commitsLast31Days;

        this.committersTotal = committersTotal;
        this.committersToday = committersToday;
        this.committersLast7Days = committersLast7Days;
        this.committersLast31Days = committersLast31Days;

        this.filesPerFileTypeChartLocator = filesPerFileTypeChartLocator;
        this.commitsPerAuthorChartLocator = commitsPerAuthorChartLocator;
        this.commitsPerFileChartLocator = commitsPerFileChartLocator;
        this.commitsPerWeekChartLocator = commitsPerWeekChartLocator;
        this.repoEntriesPerWeekChartLocator = repoEntriesPerWeekChartLocator;

        this.dirs = dirs;
        this.files = files;
        this.totalFileSize = totalFileSize;

        this.filesPerFileType = filesPerFileType;
        this.commitsPerAuthor = commitsPerAuthor;
        this.commitsPerFile = commitsPerFile;
    }

    public String getLocation() {
        return location;
    }

    public long getRevision() {
        return revision;
    }

    public List<ScmRepositoryChange> getCommits() {
        return commits;
    }

    public int getCommitsTotal() {
        return commitsTotal;
    }

    public int getCommitsToday() {
        return commitsToday;
    }

    public int getCommitsLast7Days() {
        return commitsLast7Days;
    }

    public int getCommitsLast31Days() {
        return commitsLast31Days;
    }

    public int getCommittersTotal() {
        return committersTotal;
    }

    public int getCommittersToday() {
        return committersToday;
    }

    public int getCommittersLast7Days() {
        return committersLast7Days;
    }

    public int getCommittersLast31Days() {
        return committersLast31Days;
    }

    public ZoomableImageLocator getFilesPerFileTypeChartLocator() {
        return filesPerFileTypeChartLocator;
    }

    public ZoomableImageLocator getCommitsPerAuthorChartLocator() {
        return commitsPerAuthorChartLocator;
    }

    public ZoomableImageLocator getCommitsPerFileChartLocator() {
        return commitsPerFileChartLocator;
    }

    public ZoomableImageLocator getCommitsPerWeekChartLocator() {
        return commitsPerWeekChartLocator;
    }

    public ZoomableImageLocator getRepoEntriesPerWeekChartLocator() {
        return repoEntriesPerWeekChartLocator;
    }

    public int getDirs() {
        return dirs;
    }

    public int getFiles() {
        return files;
    }

    public int getTotalFileSize() {
        return totalFileSize;
    }

    public List<Map.Entry<String,Integer>> getFilesPerFileType() {
        return filesPerFileType;
    }

    public List<Map.Entry<String,Integer>> getCommitsPerAuthor() {
        return commitsPerAuthor;
    }

    public List<Map.Entry<String,Integer>> getCommitsPerFile() {
        return commitsPerFile;
    }

    /** Returns a calculated activity percentage in the range [0.0 ... 1.0]. */
    public double getActivity() {
        // calculate activity as the ratio of commits-last-7-days and commits-average
        double commitsAverage = (double)getCommitsTotal()/getAge();
        if(commitsAverage == 0.0)
            return 0.0;

        double activity = ((double)getCommitsLast7Days()/ModelUtils.MILLIS_PER_WEEK)/commitsAverage;
        return Math.min(activity, 1.0);
    }

    public String toString() {
        return String.format("%s rev.%s", location, revision);
    }
}
